import axios from 'axios'
import debug from 'debug'
import Timer from 'tymer'
import mapzen from './adapters/mapzen'


const dbg = debug('app:geocodr')

function retryFailedRequest (err) {
  if (err.status === 504 && err.config && !err.config.__isRetryRequest) {
    err.config.__isRetryRequest = true
    return axios(err.config)
  }
  throw err
}
axios.interceptors.response.use(undefined, retryFailedRequest)

export default async function(address, opts=mapzen) {
  dbg('address=%o, opts=%o', address, opts)
  const params = {
    ...opts.params,
    [opts.addressParam]: address
  }
  dbg('params=%o', params)
  const timer = new Timer()
  const res = await axios.get(opts.url, {params})
  //dbg('geocode: res.data=%s', JSON.stringify(res.data, null, 2))
  const coordinates = opts.parse(res.data)
  dbg('geocode: address=%s, coordinates=%o, elapsed=%o', address, coordinates, timer.stop())

  return coordinates
}
